import { Observable } from 'rxjs';
import { Signal, ValueEqualityFn } from '@angular/core';
import { ObjectsComparator, ObjectsComparatorFn } from './comparator';
export type FetchedItems<T> = {
    items: T[];
    totalCount?: number;
};
export type CreateParams<T> = {
    readonly request: Observable<T> | Signal<T>;
    readonly onSuccess?: (item: T) => void;
    readonly onError?: (error: unknown) => void;
};
export type CreateManyParams<T> = {
    readonly request: Observable<FetchedItems<T> | T[]> | Observable<T>[] | Signal<FetchedItems<T> | T[]> | Signal<T>[];
    readonly onSuccess?: (items: T[]) => void;
    readonly onError?: (error: unknown) => void;
};
export type ReadParams<T> = {
    readonly request: Observable<FetchedItems<T> | T[]> | Signal<FetchedItems<T> | T[]>;
    readonly onSuccess?: (items: T[]) => void;
    readonly onError?: (error: unknown) => void;
    readonly keepExistingOnError?: boolean;
};
export type ReadOneParams<T> = {
    readonly request: Observable<T> | Signal<T>;
    readonly onSuccess?: (item: T) => void;
    readonly onError?: (error: unknown) => void;
};
export type ReadManyParams<T> = {
    readonly request: Observable<FetchedItems<T> | T[]> | Observable<T>[] | Signal<FetchedItems<T> | T[]> | Signal<T>[];
    readonly onSuccess?: (items: T[]) => void;
    readonly onError?: (error: unknown) => void;
};
export type UpdateParams<T> = {
    readonly request: Observable<T> | Signal<T>;
    readonly refreshRequest?: Observable<T> | Signal<T>;
    readonly item: Partial<T>;
    readonly onSuccess?: (item: T) => void;
    readonly onError?: (error: unknown) => void;
};
export type UpdateManyParams<T> = {
    readonly request: Observable<T[]> | Observable<T>[] | Signal<T[]> | Signal<T>[];
    readonly refreshRequest?: Observable<FetchedItems<T> | T[]> | Signal<FetchedItems<T> | T[]>;
    readonly items: Partial<T>[];
    readonly onSuccess?: (item: T[]) => void;
    readonly onError?: (error: unknown) => void;
};
export type DeleteParams<T, R = unknown> = {
    readonly request?: Observable<R> | Signal<R> | null;
    readonly item: Partial<T>;
    readonly decrementTotalCount?: boolean | number | string;
    readonly readRequest?: Observable<FetchedItems<T> | T[]> | Signal<FetchedItems<T> | T[]>;
    readonly onSuccess?: (response: R) => void;
    readonly onError?: (error: unknown) => void;
};
export type DeleteManyParams<T, R = unknown> = {
    readonly request?: Observable<R> | Observable<R>[] | Signal<R> | Signal<R>[] | null;
    readonly items: Partial<T>[];
    readonly decrementTotalCount?: boolean | number | string;
    readonly readRequest?: Observable<FetchedItems<T> | T[]> | Signal<FetchedItems<T> | T[]>;
    readonly onSuccess?: (response: R[]) => void;
    readonly onError?: (error: unknown) => void;
};
export type RefreshParams<T> = {
    readonly request: Observable<T> | Signal<T>;
    readonly item: Partial<T>;
    readonly onSuccess?: (item: T) => void;
    readonly onError?: (error: unknown) => void;
};
export type RefreshManyParams<T> = {
    readonly request: Observable<FetchedItems<T> | T[]> | Observable<T>[] | Signal<FetchedItems<T> | T[]> | Signal<T>[];
    readonly items: Partial<T>[];
    readonly onSuccess?: (item: T[]) => void;
    readonly onError?: (error: unknown) => void;
};
export type DuplicatesMap<T> = Record<number, Record<number, T>>;
export type CollectionOptions = {
    comparatorFields?: (string | string[])[];
    comparator?: ObjectsComparator | ObjectsComparatorFn;
    throwOnDuplicates?: string | false;
    allowFetchedDuplicates?: boolean;
    onDuplicateErrCallbackParam?: any;
    errReporter?: (...args: any[]) => any;
};
export type CollectionInterface<T, UniqueStatus = unknown, Status = unknown> = {
    $items: Signal<T[]>;
    $totalCountFetched: Signal<number | undefined>;
    $updatingItems: Signal<T[]>;
    $deletingItems: Signal<T[]>;
    $refreshingItems: Signal<T[]>;
    $mutatingItems: Signal<T[]>;
    $processingItems: Signal<T[]>;
    $isCreating: Signal<boolean>;
    $isReading: Signal<boolean>;
    $isUpdating: Signal<boolean>;
    $isDeleting: Signal<boolean>;
    $isMutating: Signal<boolean>;
    $isSaving: Signal<boolean>;
    $isProcessing: Signal<boolean>;
    $isBeforeFirstRead: Signal<boolean>;
    $status: Signal<Map<UniqueStatus, T>>;
    $statuses: Signal<Map<T, Set<Status>>>;
    create(params: CreateParams<T>): Observable<T>;
    createMany(params: CreateManyParams<T>): Observable<T[] | FetchedItems<T>>;
    read(params: ReadParams<T>): Observable<T[] | FetchedItems<T>>;
    readOne(params: ReadOneParams<T>): Observable<T>;
    readMany(params: ReadManyParams<T>): Observable<T[] | FetchedItems<T>>;
    refresh(params: RefreshParams<T>): Observable<T>;
    refreshMany(params: RefreshManyParams<T>): Observable<T[] | FetchedItems<T>>;
    update(params: UpdateParams<T>): Observable<T>;
    updateMany(params: UpdateManyParams<T>): Observable<T[] | FetchedItems<T>>;
    delete<R = unknown>(params: DeleteParams<T, R>): Observable<R>;
    deleteMany<R = unknown>(params: DeleteManyParams<T, R>): Observable<R[]>;
    hasItemIn(item: Partial<T>, arr: Partial<T>[]): boolean;
    uniqueItems(items: T[]): T[];
    setUniqueStatus(status: UniqueStatus, item: T, active: boolean): void;
    deleteUniqueStatus(status: UniqueStatus): void;
    setItemStatus(item: T, status: Status): void;
    deleteItemStatus(item: T, status: Status): void;
    isItemDeleting(itemSource: Partial<T> | Signal<Partial<T> | undefined>): Signal<boolean>;
    isItemRefreshing(itemSource: Partial<T> | Signal<Partial<T> | undefined>): Signal<boolean>;
    isItemUpdating(itemSource: Partial<T> | Signal<Partial<T> | undefined>): Signal<boolean>;
    isItemMutating(itemSource: Partial<T> | Signal<Partial<T> | undefined>): Signal<boolean>;
    isItemProcessing(itemSource: Partial<T> | Signal<Partial<T> | undefined>): Signal<boolean>;
    setComparator(comparator: ObjectsComparator | ObjectsComparatorFn): void;
    getDuplicates(items: T[]): DuplicatesMap<T> | null;
    getItemByPartial(partItem: Partial<T>, items: T[]): T | undefined;
    setThrowOnDuplicates(message: string | undefined): void;
    setAllowFetchedDuplicates(allowFetchedDuplicates: boolean): void;
    setOnDuplicateErrCallbackParam(onDuplicateErrCallbackParam: any): void;
    setAfterFirstReadHandler(handler: Function | undefined): void;
    getTrackByFieldFn(field: string): (i: number, item: any) => any;
    setOptions(options?: CollectionOptions | null): void;
    listenForCreate(): Observable<T[]>;
    listenForRead(): Observable<T[]>;
    listenForUpdate(): Observable<T[]>;
    listenForDelete(): Observable<Partial<T>[]>;
    listenForItemsUpdate(items: Partial<T>[]): Observable<T[]>;
    listenForItemsDeletion(items: Partial<T>[]): Observable<Partial<T>[]>;
    getItem(filter: Partial<T> | undefined | Signal<Partial<T> | undefined>, equalFn: ValueEqualityFn<T | undefined> | undefined): Signal<T | undefined>;
    getItemByField<K extends keyof T>(field: K | K[], fieldValue: T[K] | Signal<T[K]>, equalFn: ValueEqualityFn<T | undefined> | undefined): Signal<T | undefined>;
};
